/*
 * LEGAL NOTICE
 *
 * Copyright (c) 1994 WSC Investment Services, Inc.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WSC INVESTMENT SERVICES (WSCIS)
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT
 * SHALL WSCIS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR 
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS 
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Users of this software agree to return to WSCIS any improvements 
 * or extensions that they make and grant WSCIS the rights to 
 * redistribute these changes.
 *
 * Export of this software is permitted only after complying with the
 * regulations of the U.S. Deptartment of Commerce relating to the
 * Export of Technical Data.
 */

#import <objc/Object.h>
#import <hk/HKSendMail.h>

typedef enum {
    HK_LowPriority = 0,
    HK_MediumPriority = 1,
    HK_HighPriority = 2
} HKPriority;

typedef enum {
    HK_NonCriticalSeverity = 0,
    HK_SeriousSeverity = 1,
    HK_CriticalSeverity = 2
} HKSeverity;

typedef enum {
    HK_SoftwareBug = 0,
    HK_DocumentationBug = 1,
    HK_ChangeRequest = 2,
    HK_Support = 3
} HKBugClass;

    

@interface HKSendPr:Object <HKMailSender>
{
    char *emailTo;
    char *emailFrom;
    char *emailReply;

    char *submitterID;
    char *originator;
    char *organization;
    BOOL confidential;
    char *synopsis;
    HKSeverity severity;
    HKPriority priority;
    char *category;
    HKBugClass bugClass;
    char *release;

    id archInfo;

    char *description;
    char *howToRepeat;
    char *howToFix;
}


- (const char *)emailTo;
- (const char *)emailFrom;
- (const char *)emailReply;
- (const char *)submitterID;
- (const char *)originator;
- (const char *)organization;
- (BOOL)confidential;
- (const char *)synopsis;
- (HKSeverity)severity;
- (HKPriority)priority;
- (const char *)category;
- (HKBugClass)bugClass;
- (const char *)release;

- (const char *)environment;
- (const char *)system;
- (const char *)architecture;

- (const char *)description;
- (const char *)howToRepeat;
- (const char *)howToFix;


- setEmailTo:(const char *)emailAddress;
- setEmailFrom:(const char *)emailAddress;
- setEmailReply:(const char *)emailAddress;
- setSubmitterID:(const char *)aSubmitter;
- setOriginator:(const char *)anOriginator;
- setOrganization:(const char *)anOrganization;
- setConfidential:(BOOL)aFlag;
- setSynopsis:(const char *)aSynopsis;
- setSeverity:(HKSeverity)severityLevel;
- setPriority:(HKPriority)aPriority;
- setCategory:(const char *)aCategory;
- setBugClass:(HKBugClass)aBugClass;
- setRelease:(const char *)aRelease;
- setDescription:(const char *)aDescription;
- setHowToRepeat:(const char *)aDescription;
- setHowToFix:(const char *)aDescription;


- setup;
- (BOOL)send;
- writeMailToStream:(FILE *)stream;

@end

